import json
import math
import sys


class GNSSTools:
    x_pi = 3.14159265358979324 * 3000.0 / 180.0
    pi = 3.1415926535897932384626
    a = 6378245.0
    ee = 0.00669342162296594323

    @staticmethod
    def gpstobd09(lng, lat):
        [gcj_lng,gcj_lat] = GNSSTools.wgs84togcj02(lng,lat)
        [bd_lng,bd_lat] = GNSSTools.gcj02tobd09(gcj_lng,gcj_lat)
        return [bd_lng, bd_lat]


    @staticmethod
    def gcj02tobd09(lng, lat):
        z = math.sqrt(lng * lng + lat * lat) + 0.00002 * math.sin(lat * GNSSTools.x_pi)
        theta = math.atan2(lat, lng) + 0.000003 * math.cos(lng * GNSSTools.x_pi)
        bd_lng = z * math.cos(theta) + 0.0065
        bd_lat = z * math.sin(theta) + 0.006
        return [bd_lng, bd_lat]

    @staticmethod
    def bd09togcj02(bd_lon, bd_lat):
        x = bd_lon - 0.0065
        y = bd_lat - 0.006
        z = math.sqrt(x * x + y * y) - 0.00002 * math.sin(y * GNSSTools.x_pi)
        theta = math.atan2(y, x) - 0.000003 * math.cos(x * GNSSTools.x_pi)
        gg_lng = z * math.cos(theta)
        gg_lat = z * math.sin(theta)
        return [gg_lng, gg_lat]

    @staticmethod
    def wgs84togcj02(lng, lat):
        if GNSSTools.out_of_china(lng, lat):
            return lng, lat
        dlat = GNSSTools.transformlat(lng - 105.0, lat - 35.0)
        dlng = GNSSTools.transformlng(lng - 105.0, lat - 35.0)
        radlat = lat / 180.0 * GNSSTools.pi
        magic = math.sin(radlat)
        magic = 1 - GNSSTools.ee * magic * magic
        sqrtmagic = math.sqrt(magic)
        dlat = (dlat * 180.0) / ((GNSSTools.a * (1 - GNSSTools.ee)) / (magic * sqrtmagic) * GNSSTools.pi)
        dlng = (dlng * 180.0) / (GNSSTools.a / sqrtmagic * math.cos(radlat) * GNSSTools.pi)
        mglat = lat + dlat
        mglng = lng + dlng
        return [mglng, mglat]

    @staticmethod
    def gcj02towgs84(lng, lat):
        if GNSSTools.out_of_china(lng, lat):
            return lng, lat
        dlat = GNSSTools.transformlat(lng - 105.0, lat - 35.0)
        dlng = GNSSTools.transformlng(lng - 105.0, lat - 35.0)
        radlat = lat / 180.0 * GNSSTools.pi
        magic = math.sin(radlat)
        magic = 1 - GNSSTools.ee * magic * magic
        sqrtmagic = math.sqrt(magic)
        dlat = (dlat * 180.0) / ((GNSSTools.a * (1 - GNSSTools.ee)) / (magic * sqrtmagic) * GNSSTools.pi)
        dlng = (dlng * 180.0) / (GNSSTools.a / sqrtmagic * math.cos(radlat) * GNSSTools.pi)
        mglat = lat + dlat
        mglng = lng + dlng
        return [lng * 2 - mglng, lat * 2 - mglat]

    @staticmethod
    def transformlat(lng, lat):
        ret = -100.0 + 2.0 * lng + 3.0 * lat + 0.2 * lat * lat + \
                0.1 * lng * lat + 0.2 * math.sqrt(math.fabs(lng))

        ret += (20.0 * math.sin(6.0 * lng * GNSSTools.pi) + 20.0 *
                math.sin(2.0 * lng * GNSSTools.pi)) *2.0 / 3.0
        ret += (20.0 * math.sin(lat * GNSSTools.pi) + 40.0 *
                math.sin(lat / 3.0 * GNSSTools.pi)) *2.0 / 3.0
        ret += (160.0 * math.sin(lat / 12.0 *GNSSTools. pi) + 320 *
                math.sin(lat * GNSSTools.pi / 30.0)) *2.0 / 3.0
        return ret

    @staticmethod
    def transformlng(lng, lat):
        ret = 300.0 + lng + 2.0 * lat + 0.1 * lng * lng + \
                0.1 * lng * lat + 0.1 * math.sqrt(math.fabs(lng))

        ret += (20.0 * math.sin(6.0 * lng * GNSSTools.pi) + 20.0 *
                math.sin(2.0 * lng * GNSSTools.pi)) *2.0 / 3.0
        ret += (20.0 * math.sin(lng *GNSSTools.pi) + 40.0 *
                math.sin(lng / 3.0 * GNSSTools.pi)) *2.0 / 3.0
        ret += (150.0 * math.sin(lng / 12.0 * GNSSTools.pi) + 300.0 *
                math.sin(lng / 30.0 * GNSSTools.pi)) *2.0 / 3.0
        return ret

    @staticmethod
    def out_of_china(lng, lat):
        if lng < 72.004 or lng > 137.8347:
            return True
        if lat < 0.8293 or lat > 55.8271:
            return True
        return False

    @staticmethod
    def processJson(inputJsonFile, outputJsonFile):
        file_grid = outputJsonFile + "_grid.json"

        file_heat = outputJsonFile + "_heat.json"
        fin = open(inputJsonFile, 'r')
        fout = open(file_heat, 'w')
        fout_ = open(file_grid, 'w')

        out = []
        out_ = []
        for eachLine in fin:
            fileName, leftLon, rightLon, topLat, bottomLat, cnt, color = eachLine.split(",")
        data = {}
        data_ = {}
        result1 = GNSSTools.wgs84togcj02(float(leftLon), float(topLat))
        result2 = GNSSTools.gcj02tobd09(result1[0], result1[1])
        result3 = GNSSTools.wgs84togcj02(float(rightLon), float(bottomLat))
        result4 = GNSSTools.gcj02tobd09(result3[0], result3[1])
        data["lat"] = result2[1]
        data["lng"] = result2[0]
        data["count"] = cnt
        out += [data]


        data_["topLat"] = result2[1]
        data_["leftLon"] = result2[0]
        data_["bottomLat"] = result4[1]
        data_["rightLon"] = result4[0]
        data_["cnt"] = cnt
        data_["color"] = color
        out_ += [data_]


        fout.write(str(out))
        fout_.write(str(out_))
        fin.close()
        fout.close()
        fout_.close()

    @staticmethod
    def processCSV(inputJsonFile, outputJsonFile):
        file_csv = outputJsonFile + ".csv"

        fin = open(inputJsonFile, 'r')
        fout = open(file_csv, 'w')

        for eachLine in fin:
            fileName, addr, topLat, leftLon = eachLine.split(",")
        result1 = GNSSTools.bd09togcj02(float(leftLon), float(topLat))
        result2 = GNSSTools.gcj02towgs84(result1[0], result1[1])
        out = str(fileName) + "," + str(addr) + "," + str(result2[0]) + "," + str(result2[1]) + "\n"
        fout.write(str(out))
        fin.close()
        fout.close()

if __name__ == '__main__':
    ll=GNSSTools.gpstobd09(116.314049758,40.069384452)
    print('%f,%f' % (ll[0],ll[1]))


import consul
import json
import uuid
import logging

class ConsulService(object):
    def __init__(self, host, port):
        '''初始化，连接consul服务器'''
        self.host = host
        self.port = port
        self._consul = consul.Consul(host, port)

    def __buildApiConfiguation(self, local_ip, port, app_name, app_id, prefix, app_group, instance_info, apis, protocol='http',health_entry='/health'):
        tags = []

        tags.append("group={}".format(app_group))
        tags.append("secure=false")
        tags.append("{}.enable=true".format(prefix))
        tags.append("{}.http.services.{}.loadbalancer.server.weight=1".format(prefix, app_name))
        tags.append("{}.http.services.{}.loadbalancer.passHostHeader=true".format(prefix, app_name))
        tags.append("{}.http.services.{}.loadbalancer.server.scheme={}".format(prefix, app_name,protocol))
        tags.append("{}.instance={}".format(app_id, json.dumps(instance_info)))

        for api in apis:
            uid = str(uuid.uuid4())
            suid = ''.join(uid.split("-"))
            route_prefix = "{}.http.routers.{}".format(prefix, suid)
            tags.append("{}.rule=Path(`{}`) && Method(`{}`)".format(route_prefix, api['path'], api['method']))
            key = None
            if 'scope' in api.keys():
                key = 'scope'
            elif 'scopes' in api.keys():
                key = 'scopes'
            if key is not None:
                tags.append("{}.middlewares=nezha@internal".format(route_prefix))
                tags.extend(
                    ["{}.metadata.nezha.scopes={}".format(route_prefix, scope) for scope in api[key].split(',')])

        logging.info("Tags:" + str(tags))
        api_info={"tags": tags,"local_ip":local_ip,"port":port,"app_id":app_id,"app_name":app_name,"app_group":app_group,"instance_info":instance_info,"protocol":protocol,"health_entry":health_entry}
        return api_info

    """
    def registerAPI(self, data):
        #data=self.buildApiConfiguation(local_ip,port,app_name,app_id,perfix,app_group,instance_info,apis,protocol,health_entry)

        self.registerService(data['app_name'], data['app_id'], data['local_ip'], data['port'], tags=data['tags'], protocol=data['protocol'],
                             health_entry=data['health_entry'])

        svc,res = self.getService(data['app_id'])
        logging.info(str(svc))
    """

    def registerService(self, name, service_id, host, port, tags=None, protocol='http', health_entry='/health'):
        tags = tags or []
        url = "{}://{}:{}{}".format(protocol, host, port, health_entry)
        logging.info("Health Url: {}".format(url))
        # 注册服务
        self._consul.agent.service.register(
            name=name,
            service_id=service_id,
            address=host,
            port=port,
            tags=tags,
            # 健康检查ip端口，检查时间：5,超时时间：30，注销时间：30s
            check=consul.Check().http(url, "5s", "30s", "30s", None))

    def check(self, host, port, interval, timeout=None, deregister=None, header=None, health_entry='/health',
              protocol='http'):
        return consul.Check.http("{}}://{}:{}{}".format(protocol, host, port, health_entry), interval, timeout,
                                 deregister, header)

    def getService(self, servie_id):
        services = self._consul.agent.services()
        service = services.get(servie_id)
        if not service:
            return None, None
        addr = "{0}:{1}".format(service['Address'], service['Port'])
        return service, addr

    def registerNezhaApi(self,settings,api_entries,instance_info):
        local_ip = Utility.getLocalIP(host=self.host,port=self.port)
        port = settings.port
        app_name = settings.name
        app_id = '{}-{}'.format(app_name, local_ip.replace(".", "-"))
        prefix = "traefik"
        tags = []

        tags.append("group={}".format(settings.consul.instance_group))
        tags.append("secure=false")
        tags.append("{}.enable=true".format(prefix))
        tags.append("{}.http.services.{}.loadbalancer.server.weight=1".format(prefix, app_name))
        tags.append("{}.http.services.{}.loadbalancer.passHostHeader=true".format(prefix, app_name))
        if instance_info is not None:
            tags.append("{}.instance={}".format(app_id, json.dumps(instance_info)))

        for api in api_entries:
            uid = str(uuid.uuid4())
            suid = ''.join(uid.split("-"))
            api_prefix = '{}.http.routers.{}'.format(prefix, suid)
            tags.append("{}.rule=Path(`{}`) && Method(`{}`)".format(api_prefix, api['path'], api['method']))
            if ('scope' in api.keys()):
                tags.append("{}.metadata.nezha.scopes={}".format(api_prefix, api['scope']))
                tags.append("{}.middlewares=nezha@internal`)".format(api_prefix))

       
        res = self.registerService(app_name, app_id, local_ip, port, tags)

        # check=consul_service.check()
        # print(check)
        res = self.getService(app_id)
        print(res)

if __name__ == '__main__':
    import sys
    sys.path.append('../../')
    from inhand.service.SettingService import SpringCloudSettings, Consul

    from inhand.utilities.Utility import Utility
    settings=SpringCloudSettings()
    settings.readBootstrapYaml("d:\\bootstrap.yml")

    api_entries = [
        {
            "desc": "获取老化车信息",
            "path": "/api/v1.0/aging_car/info",
            "method": "GET",
            "scope": "mes:aging:get"
        },
        {
            "desc": "获取老化信息",
            "path": "/api/v1.0/aging/query",
            "method": "GET",
            "scope": "mes:aging:get"
        },
        {
            "desc": "序列号绑定老化车",
            "path": "/api/v1.0/aging/sn_bind",
            "method": "POST",
            "scope": "mes:aging:post"
        },
        {
            "desc": "工单绑定老化车",
            "path": "/api/v1.0/aging/order_bind",
            "method": "POST",
            "scope": "mes:aging:post"
        }
    ]

    consulService = ConsulService(settings.consul.host, settings.consul.port)
    consulService.registerNezhaApi(settings,api_entries,None)
    print("done")
    #with open('./api-info.json','w') as f:
    #    json.dump(data,f)

    #with open('./api-info.json','r') as f:
    #    data1=json.load(f)
    #consulService.registerAPI(data1)



    """
    tags=[]
    tags.append("group=inhand-poweris")
    tags.append("secure=flaes")
    tags.append("traefik.enable=true")

    tags.append("traefik.http.services.jx-attendance-api.loadbalancer.server.weight=1")
    consulService=ConsulService("consul",8500)
    name="jx-attandance-api-10-0-0-1"
    service_id="jx-attendance-api"
    consulService.registerService(name,service_id,"10.5.0.247",5000,tags)

    check=consulService.check("10.5.0.247",5000,"30s")
    print(check)
    res=consulService.getService(service_id)
    print(res)
    """





#encoding=utf-8
import pika
import logging,traceback

class RabbitMQ:
    def __init__(self,mq_settings):
        self.settings=mq_settings
        self.conn=None

    def __connect(self):
        if self.conn is None:
            credentials = pika.PlainCredentials(self.settings.account, self.settings.password)
            self.conn=pika.BlockingConnection(pika.ConnectionParameters(self.settings.host,self.settings.port,'/',credentials))

    def send_msg(self,exchange,routing_key,message):
        try:
            self.__connect()
            channel = self.conn.channel()
            #channel.exchange_declare(exchange=exchange, durable=True, exchange_type='fanout')
            #channel.basic_publish(exchange=exchange, routing_key=routing_key, body=message,  mandatory=True,properties=pika.BasicProperties(delivery_mode = 2))

            channel.basic_publish(exchange=exchange, routing_key=routing_key, body=message)
            self.conn.close()
            self.conn = None
            return 200
        except Exception as e:
            traceback.print_exc()
            logging.warning(e,'while sending message to exchange: {} with key: {}'.format(exchange,routing_key))
            try:
                self.conn.close()
            finally:
                self.conn=None
                return 500


    """
credentials = pika.PlainCredentials('admin', '1qaz2wsx')
connection = pika.BlockingConnection(pika.ConnectionParameters('10.5.16.213',5672,'/',credentials))

exchange='exchange.iot'
routing=sys.argv[3]#'device.5b865291b9d8210005d679cc.shadow.update.resp'
vid=sys.argv[1]
count=sys.argv[2]
channel = connection.channel()
#channel.exchange_declare(exchange=exchange)
#channel.queue_declare('smartfleet.condition')

#message = '{"timestamp": "2016-10-1'+str(count)+'T22:00:00","content": "PLC1温度高过60摄氏度了","type": "E123","group": "temperature", "level": 2,"state": "on"}'

message='[{"fields": {"altitude": 0.0,"latitude": 40.067412,"speed": 0.0,"longitude": 116.319018},"metric": "location_data","tags": {'\
            +'"mid": "5b911037ea40310005350e09","category": "gps","did": "5b865291b9d8210005d679cc"},"ts": "2018-09-10T17:09:59+0800"}]'
#message = '{"trans_id":'+str(count)+',"address":"ç´¢é©¬é","createDate": ' +str(int(time.time())) +',"p":{"coordinates":{23:25},"type":"POINT"},"status":{"cd":"cidg"},"vid":"'+vid+'"}'
channel.basic_publish(exchange=exchange,routing_key=routing,body=message)
print(" [x] ->"+message)
connection.close()

    """


# 引入Minio包。
import json

from botocore.exceptions import ClientError
import logging

class OSSServiceFactory:
    @staticmethod
    def creatStorageProvider(provider:str,cateogry:str,settings:dict):
        if cateogry == 's3' or cateogry == 'aws':
            return AWSS3Service(provider,settings)
        elif cateogry == 'minio':
            return MinoService(provider,settings)
        else:
            raise Exception("Unkonw CloudFileService")


class CloudFileService:

    def __init__(self,provider:str,settings:dict):
        self.setttings=settings
        self.provider=provider
        self.client=None

    def upload(self,bucket:str,src_path:str,dst_path:str,metadata:dict):
        #raise Exception("Can not call the abstract method")
        return {"bucket":bucket,"key":dst_path,"provider":self.provider}



class AWSS3Service(CloudFileService):
    """
    https://boto3.amazonaws.com/v1/documentation/api/latest/guide/s3-uploading-files.html
    """
    def __init__(self,provider:str,settings:dict):
        super().__init__(provider,settings)
        self.endpoint = settings['endpoint']
        self.access_key = settings['access_key']
        self.secret_key = settings['secret_key']
        self.secure = settings['secure']
        self.bucket=settings['bucket']
        self.client=None

    def __connect__(self):
        import boto3
        try:
            self.client = boto3.client(
                's3',
                aws_access_key_id=self.access_key,
                aws_secret_access_key=self.secret_key,
                endpoint_url=self.endpoint,
                verify=False
            )

        except ClientError as err:
            raise err

    def upload(self,bucket:str,src_path:str,dst_path:str,metadata:dict):
        self.__connect__()
        try:

            content_type = "application/octet-stream" if "content-type" not in metadata.keys() else metadata['content-type']
            response = self.client.put_object(Bucket=bucket, Body=open(src_path, 'rb'),
                                              Key=dst_path,
                ContentType=content_type,
                Metadata=metadata)
            """
            {'ResponseMetadata': {'RequestId': 'FEASEX6M9V2V2E5Q', 'HostId': 'FsSozf3bTabOIrmNIvZnnwRjbDpy/uKKTqnyWMLWVpg3uOpor+3TJvQqWIxkSzrMv5gBWPvEiadW45tKzt2OcA==', 'HTTPStatusCode': 200, 'HTTPHeaders': {'x-amz-id-2': 'FsSozf3bTabOIrmNIvZnnwRjbDpy/uKKTqnyWMLWVpg3uOpor+3TJvQqWIxkSzrMv5gBWPvEiadW45tKzt2OcA==', 'x-amz-request-id': 'FEASEX6M9V2V2E5Q', 'date': 'Sun, 19 Mar 2023 15:32:00 GMT', 'x-amz-server-side-encryption': 'AES256', 'etag': '"8983b25e38ccb97b45921d966f0998d8"', 'server': 'AmazonS3', 'content-length': '0'}, 'RetryAttempts': 1}, 'ETag': '"8983b25e38ccb97b45921d966f0998d8"', 'ServerSideEncryption': 'AES256'}
            """
            if response['ResponseMetadata']['HTTPStatusCode'] == 200:
                return {"result":{"etag":response['ETag'],"bucket":bucket,"key":dst_path,"provider":self.provider},"status":200}
            else:

                # return {"result": "error", "status": response['ResponseMetadata']['HTTPStatusCode'], "error": json.dump(response)}
                raise Exception("Upload file to s3 failed: {}".format(json.dumps(response)))
        except Exception as e:
            logging.error(e)

            return {"result":"error","status":500,"error":str(e)}





class MinoService(CloudFileService):
    def __init__(self, provider:str,settings: dict):
        super().__init__(provider,settings)
        self.endpoint = settings['endpoint']
        self.access_key = settings['access_key']
        self.secret_key = settings['secret_key']
        self.secure = settings['secure']
        self.bucket=settings['bucket']
        self.client = None


    def __connect__(self):

        from minio import Minio
        if self.client is None:
            try:
                self.client = Minio(self.endpoint,
                                    access_key=self.access_key,
                                    secret_key=self.secret_key,
                                    secure=False)

            except Exception as err:
                raise err

    def upload(self,bucket:str,src_path:str,dst_path:str,metadata:dict):
        self.__connect__()
        import os
        try:
            with open(src_path, 'rb') as file_data:
                file_stat = os.stat(src_path)
                rsp=self.client.put_object(
                    bucket,
                    dst_path,
                    file_data,
                    file_stat.st_size,
                    metadata=metadata
                )
                return {"result":{"etag":rsp.etag,"bucket":rsp.bucket_name,"key":rsp.object_name,"provider":self.provider},"status":200}

        except ClientError as e:
            logging.error(e)
            return {"result": "error", "status": 500,"error": str(e)}


if __name__ == '__main__':

    metadata={"filename":"a.json","content-type":"application/json","content-type": "application/octet-stream"}
    src_file= "/inhand-product-ci.sample"
    key="2023-03-19/{}".format(src_file[src_file.rindex("/")+1:])


    minio_settings={"access_key":"protected-user","secret_key":"rnk4gVRMxojVXkv","endpoint":"files.inhand.design","secure":False}
    minio_svc=OSSServiceFactory.creatStorageProvider("cn.product.protected.resources.","minio",minio_settings)
    rsp=minio_svc.upload("protected-files",src_file,key,metadata)
    print("response: {}".format(rsp))


    s3_settings={"access_key":"AKIAYFRJLY5HJZKFTKAN",
                 "secret_key":"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",
                 "endpoint":"https://s3.cn-north-1.amazonaws.com.cn",
                 "region":"cn-north-1",
                 "secure":False}
    s3_svc=OSSServiceFactory.creatStorageProvider("cn.product.protected.resources","s3",s3_settings)

    rsp=s3_svc.upload("internet-protected-files",src_file,key,metadata)
    print("response: {}".format(rsp))











ERR_CODE_400 = 400
ERR_CODE_404 = 404
ERR_CODE_401 = 401
ERR_CODE_403 = 403
ERR_CODE_500 = 500
ERR_CODE_502 = 502

class Error:
    def __init__(self,errorCode, desc):
        self.error_code = errorCode
        self.error = desc


import http.client
import json
import time
import ssl
import requests
from datetime import datetime
from urllib import parse
import logging,traceback

class CalendarService:
    def __init__(self,settings):
        import sys
        sys.path.append('../../')
        from inhand.utilities.URLUtility import GenericURL
        self.settings = settings
        ssl._create_default_https_context = ssl._create_unverified_context


    # 月份
    def query(self,year,month):
        raise  NameError('This is a parent class, there is nothing to do!')


    @staticmethod
    def createCalendarService(settings):
        #if settings.app.calendar_provider == 'baidu':
        #    return BaiduCalendarService(settings)
        if settings.calendar_provider == 'wannianli':
            return WanNianLi(settings)
        else:
            return None




class WanNianLi:
    def __init__(self,settings):
        CalendarService.__init__(self,settings)
        self.calendar_provider = self.settings.calendar_provider
        self.calendar_url = self.settings.calendar_url

#    def __init__(self,provider,url):
#        self.calendar_provider = provider
#        self.calendar_url = url


    # 输入参数：月份
    # 参考： https://www.mxnzp.com/doc/detail?id=1
    def query(self, year, month):
        ym = datetime.now().strftime('%Y%m')
        if year is not None and month is not None:
            ym = "{}{}".format(year,str(month).zfill(2))
        url=self.calendar_url.format(ym)
        days={}
        try:
            r=requests.get(url)
            f = r.json()
            for item in f['data']:
                #type; 类型 0 工作日 1 假日 2 节假日 如果ignoreHoliday参数为true，这个字段不返回
               days[item['date']] = 1 if item['type'] != 0 else 2

            return days

        except Exception as e:
            traceback.print_exc()
            logging.warning("While call %s, There is an exception: %s" % (self.calendar_url, e))
            return None


"""
if __name__ == '__main__':
    import sys

    #sys.path.append('../../')
    #from inhand.service.SettingService import SpringCloudSettings
    #settings=SpringCloudSettings()
    #settings.readLocalSettings(path="../../../../test.json",module="jx-attendance-stat-svc")

    #calcendar = CalendarService.createCalendarService(settings)
    calcendar=WanNianLi("wangnianli","https://www.mxnzp.com/api/holiday/list/month/{}?app_id=j89kjqhkrnxtlorm&app_secret=c1lkY0dneElHNDdtQVlZWTRkNjlGZz09")
    ret = calcendar.query('2022','04')
    print(ret)
    ret = calcendar.query(2022,4)
    print(ret)
"""



from urllib.parse import urlparse

class GenericURL:
    def __init__(self,url):
        self.url = url
        parsed = urlparse(url)
        self.scheme = parsed.scheme
        self.host = parsed.netloc
        self.path = parsed.path
        self.params=GenericURL.parseParams(parsed.query)

    @staticmethod
    def parseParams(query):
        params = {}
        if query is not None and '' != query:
            list = query.split("&")
            for s in list:
                index = s.index("=")
                if index > -1:
                    params[s[0:index]] = s[index+1:]
        return params

if __name__ == '__main__':
    str="http://api.map.baidu.com:8801/geocoder/v2/d?ad=34&ak=wo5GfWYQ0VEhLn2ZRfl1v3l0auM2MnaR&b=1"
    url = GenericURL(str)
    print(url.path)


import uuid


class ApiInfo:
    def __init__(self,path,method,scope=None):
        self.path=path
        self.method=method.upper()
        self.scope=scope
        suid = str(uuid.uuid4())
        self.uid= ''.join(suid.split("-"))

    def createApiEntry(self,prefix):
        api_prefix = '{}.http.routers.{}'.format(prefix, self.uid)
        if self.scope is not None:
            tags=[]
            tags.append("{}.rule=Path(`{}`) && Method(`{}`)".format(api_prefix, self.path, self.method))
            if self.scope is not None and '' != self.scope:
                tags.append("{}.metadata.nezha.scopes={}".format(api_prefix,self.scope))
                tags.append("{}.middlewares=nezha@internal`)".format(api_prefix))
                return tags
            else:
                return []



    @staticmethod
    def parseApiController(file):
        #ToDo: 读取fast api 中controller，解析得到ApiInfo的集合
        pass

class ResourcAPIPath:
    def __init__(self):
        self.paths={}

    def path(self,path:str):
        def wrapper(cls):
            print(cls)

        return wrapper

class RestfulAPI:
    def __init__(self):
        self.apis={}

    def api(self, path:str,**kwds):
        def decorate(fn):
            method=None
            for item in kwds.items():
                key = item[0]
                value = item[1]
                http_path=path #('%s%s' % (self.root_path,'' if path is None else path))
                http_scope=None
                if key == 'scope':
                    http_scope = value
                elif key == 'method':
                    method = value
            http_method = fn.__name__ if method is None else method

            cls_name_index=fn.__qualname__.index(".")
            cls_name=fn.__qualname__[:cls_name_index]
            self.add_api(cls_name,http_path,http_method,http_scope)

            print('======fun.path==== %s' % http_path)
            return fn
        return decorate

    def add_api(self,cls_name,http_path,http_method,http_scope):
        api_info=ApiInfo(http_path,http_method,http_scope)
        if cls_name not in self.apis.keys():
            self.apis[cls_name]=[]
        self.apis[cls_name].append(api_info)



api = RestfulAPI()
paths=ResourcAPIPath()

#@ResourcAPIPath.path('/api/demo')
class demo:
    @api.api(method='GET', path='demo1/{id}', scope='a:b:c')
    def get(self):
        print('yes')

    @api.api(path='demo1', scope='a:b:d')
    def post(self):
        print('yes')

#@ResourcAPIPath.path('/api/demo1')
class demo2:
    @api.api(method='GET', path='demo2/{id}', scope='a:b:c')
    def get(self):
        print('yes')

    @api.api(path='demo/2', scope='a:b:d')
    def post(self):
        print('yes')
if __name__ == '__main__':
    print(api)
    print('-------')




#coding=utf-8
from hdfs.client import Client
#from hdfs import Config


class HdfsService:
    def __init__(self,url,root):
        self.url=url
        self.client=None
        self.root=root if root is not None else '/'


    def __connect__(self,path=None):
        if self.client is None:
            #config=hdfs.config.Config()
            #self.client=config.get_client()
            self.client=Client(self.url,root=self.root,timeout=60000)
            #self.client = KerberosClient(self.url, root="/")



    # 读取hdfs文件内容,将每行存入数组返回
    def cat(self, filename):
        # with client.read('samples.csv', encoding='utf-8', delimiter='\n') as reader:
        #  for line in reader:
        # pass
        self.__connect__()
        lines = []
        with self.client.read(filename, encoding='utf-8', delimiter='\n') as reader:
            for line in reader:
                # pass
                # print line.strip()
                lines.append(line.strip())
        return lines

    # 创建目录
    def mkdir(self, hdfs_path):
        self.__connect__()
        self.client.makedirs(hdfs_path)

    # 删除hdfs文件
    def rm(self, hdfs_path):
        self.__connect__()
        self.client.delete(hdfs_path)

    # 上传文件到hdfs
    def put(self,local_path, hdfs_path):
        self.__connect__()
        self.client.upload(hdfs_path, local_path)

    # 从hdfs获取文件到本地
    def get(self, hdfs_path, local_path):
        self.__connect__()
        self.client.download(hdfs_path, local_path, overwrite=False)

    # 追加数据到hdfs文件
    def append(self, hdfs_path, data):
        self.__connect__()
        self.client.write(hdfs_path, data, overwrite=False, append=True, encoding='utf-8')

    # 覆盖数据写到hdfs文件
    def write(self, hdfs_path, data):
        self.__connect__()
        self.client.write(hdfs_path, data, overwrite=True, append=False, encoding='utf-8')

    # 移动或者修改文件
    def mv(self, hdfs_src_path, hdfs_dst_path):
        self.__connect__()
        self.client.rename(hdfs_src_path, hdfs_dst_path)

    # 返回目录下的文件
    def ls(self, hdfs_path):

        self.__connect__()
        return self.client.list(hdfs_path, status=False)

if __name__ == '__main__':
    svc=HdfsService('http://wyx.inhandcloud.com:9870',root="/")
    result=svc.ls("/")
    print(result)
    svc.mkdir("/tube_data2")
    result=svc.ls("/tube_data")
    print(result)
    #hdfs.put('/Users/han/temp/jquery-validation-1.19.1.zip','/tube_data')
    svc.put('/home/han/temp/gehc_ib.1595030785.csv','/tmp')
    result=svc.ls("/tube_data")
    print(result)



class BasicResult:
    def __init__(self,error=None,error_code=None,result=None):
        self.error=error
        self.error_code=error_code
        self.result=result
        self.status=error_code if error_code is not None else 200

class PageResult:
    def __init__(self,error=None,error_code=None,total=0,pageNumber=1,pageSize=None,result=None,stats=None):
        self.error=error
        self.error_code=error_code
        self.result=result
        self.total=total
        self.pageNumber=pageNumber
        self.pageSize=pageSize
        self.stats=stats
        self.status = error_code if error_code is not None else 200



# 基类注解
def base(annotation):
    def decorator_annos(*args, **kwargs):
        # print(args,kwargs)
        def decorator(fn):
            sub_annos_name = annotation.__name__
            if fn.__annotations__.get(sub_annos_name) is None:
                # 以子类注解名作为key生成一个字典，用于区分注解
                fn.__annotations__[sub_annos_name] = {}
            # 放钩子，调用子类注解
            annotation(fn, *args, **kwargs)
            return fn
        return decorator
    return decorator_annos

@base
def annotationA(fn, *args, **kwds):
    if args != None and len(args) != 0:
        if len(args) > 1:
            raise RuntimeError("nums of @annotationA parameter can only be one! eg. @annotationA('功能A')")
        fn.__annotations__["annotationA"]["value"] = args[0]

    elif len(kwds) != 1 or (not kwds.keys().__contains__("value")):
        raise RuntimeError("If the form of @annotationA parameter is k,v \
    , then key can only be 'value' str! eg. @annotationA(value='功能A')")

    else:
        for k, v in kwds.items():
            fn.__annotations__["annotationA"][k] = v


@annotationA("this is A function")
def test_fun1():
    pass


@annotationA(value="this is A function")
def test_fun2():
    print('-------------')
"""
————————————————
版权声明：本文为CSDN博主「青春依旧_」的原创文章，遵循CC
4.0
BY - SA版权协议，转载请附上原文出处链接及本声明。
原文链接：https: // blog.csdn.net / qq_37960007 / article / details / 115017587
"""

if __name__ == '__main__':
    #assert test_fun1.__annotations__ == test_fun2.__annotations__
    #print(test_fun1.__annotations__)
    #print(test_fun2.__annotations__)

    test_fun2()


import json
import datetime




class Metadata:
    created_at: datetime = None
    updated_at: datetime = None
    created_by: str = None
    updated_by: str = None

    def __init__(self, created_by: str = None, updated_by: str = None):
        if created_by is not None:
            self.created_by = created_by;
            self.created_at = datetime.now()
        elif updated_by is not None:
            self.updated_by = updated_by
            self.updated_at = datetime.now()

    def toDict(self):
        kv = {}
        if self.updated_by is not None:
            kv['updated_by'] = self.updated_by
            kv['updated_at'] = self.updated_at
        else:
            kv['created_by'] = self.created_by
            kv['created_at'] = self.created_at
        return kv

    @staticmethod
    def fromDict(kv: dict):
        metadata = Metadata()
        metadata.created_by = None if ['created_by'] not in kv.keys() else kv['created_by']
        metadata.created_at = None if ['created_at'] not in kv.keys() else kv['created_at']
        metadata.updated_by = None if ['updated_by'] not in kv.keys() else kv['updated_by']
        Metadata.updated_at = None if ['updated_at'] not in kv.keys() else kv['updated_at']
        return metadata

    def tostring(self):
        json_string = json.dumps(self.toDict());
        return json_string

class Model:
    metadata: Metadata = None

    def updatedby(self, by: str):
        if self.metadata is None:
            self.metadata = Metadata(created_by=by, updated_by=by)
        else:
            self.metadata.updated_by = by
            self.metadata.updated_at = datetime.now()


#coding=utf-8
from datetime import datetime,timedelta,date
from dateutil import parser
from tzwhere import tzwhere
import time
import pytz
import pandas as pd
import rfc3339
import iso8601


class DatetimeUtility:
    @staticmethod
    def retrieveLocalTimeOffset():
        return 0-time.timezone/3600

    @staticmethod
    def retrieveTimeZone(longitude,latitude):
        tz = tzwhere.tzwhere()
        return tz.tzNameAt(latitude,longitude)

    @staticmethod
    def retrieveTimeoffset(timezone=None):

        if timezone is None or timezone == '':
            return DatetimeUtility.retrieveLocalTimeOffset()
        else:
            try:
                tz = pytz.timezone(timezone)
                ss = int(tz._utcoffset.total_seconds() / 3600)
                return ss
            except Exception as e:
                print(e)
                return DatetimeUtility.retrieveLocalTimeOffset()

    #返回当天的0点和第二天的0点两个时间
    @staticmethod
    def day_scope(date_str,):
        date = datetime.datetime.strptime(date_str,'%Y-%m-%d')
        return
    #返回从开始到结束时间的日期字符串series
    @staticmethod
    def date_range(timezone,start,end):
        period =pd.date_range(start,end-timedelta(days=1),freq='1d',tz=timezone)
        return period

    @staticmethod
    def calcTheDayRange(timezone=None,date_str=''):
        tzoffset = DatetimeUtility.retrieveTimeoffset(timezone=timezone)
        timestr = ('%sT00:00:00Z' % date_str)
        theDay = iso8601.parse_date(timestr).astimezone(pytz.UTC)+timedelta(hours=tzoffset)
        start = theDay.replace(hour=0, minute=0, second=0, microsecond=0)-timedelta(hours=tzoffset)
        end = theDay.replace(hour=0, minute=0, second=0, microsecond=0)+timedelta(days=1)-timedelta(hours=tzoffset)
        return (start,end)

    @staticmethod
    def calcDateRange(timezone=None,s=None):

        tzoffset = DatetimeUtility.retrieveTimeoffset(timezone=timezone)
        if s is None:
            s=datetime.now(tz=pytz.UTC)
        else:
            if s.tzinfo is not None:
                s=s.astimezone(tz=pytz.UTC)
            else:
                s=s.replace(tzinfo=pytz.UTC)
        e=datetime.now(tz=pytz.UTC)
        #e=e-timedelta(hours=tzoffset)
        start = s.replace(hour=0, minute=0, second=0, microsecond=0)-timedelta(hours=tzoffset)
        end = e.replace(hour=0, minute=0, second=0, microsecond=0)-timedelta(hours=tzoffset)

        return (start,end)


    @staticmethod
    def calcCurMonthRange(timezone=None):
        tzoffset = DatetimeUtility.retrieveTimeoffset(timezone=timezone)
        s = datetime.now(tz=pytz.UTC)
        start = s.replace(day=1,hour=0, minute=0, second=0, microsecond=0)-timedelta(hours=tzoffset)
        end = s.replace(hour=0, minute=0, second=0, microsecond=0)-timedelta(hours=tzoffset)
        return (start,end)

    @staticmethod
    def calcCurMonthRange2Ts(timezone=None):
        #time.mktime(datetime.today().timetuple())
        (start,end) = DatetimeUtility.calcCurMonthRange(timezone=timezone)
        diff = end - start
        if diff.days == 0:
            return (None, None)
        else:
            return (DatetimeUtility.toUnixTs(dt=start),DatetimeUtility.toUnixTs(dt=end))

    @staticmethod
    def toUnixTs(timezone=None,dt=None):
        if dt is None:
            return 0
        else:
            if dt.tzinfo is not None:
                dst = dt.astimezone(pytz.UTC)
            else:
                dst = dt.replace(tzinfo=pytz.UTC)
            seconds = DatetimeUtility.retrieveTimeoffset(timezone=timezone) * 3600
            return time.mktime(dst.timetuple())+seconds

    @staticmethod
    def toISO8601(dt):

        if dt.tzinfo is not None:
            dt = dt.astimezone(pytz.UTC)
            return dt.strftime('%Y-%m-%dT%H:%M:%SZ')
        else:
            dt = dt.replace(tzinfo=pytz.UTC)
            return dt.strftime('%Y-%m-%dT%H:%M:%SZ')
            #return ('%sZ' % dt.isoformat())

    # 把datetime转成字符串
    @staticmethod
    def toDateString(timezone=None,dt=None):

        tzoffset = DatetimeUtility.retrieveTimeoffset(timezone=timezone)

        return (dt+timedelta(hours=tzoffset)).strftime("%Y-%m-%d")

    @staticmethod
    def calcLast24Hours():
        dt = datetime.now(pytz.UTC)

        end = dt.replace(minute=0, second=0,microsecond=0)
        if dt.minute <=5:
            end= dt.replace(minute=0, second=0, microsecond=0) - timedelta(hours=1)
        start = end - timedelta(days=1)
        return (start,end)


    @staticmethod
    def calcLast2Now(start=None):
        dt = datetime.now(pytz.UTC)

        end = dt.replace(minute=0, second=0,microsecond=0)
        if dt.minute <=1:
            end= dt.replace(minute=0, second=0,microsecond=0) - timedelta(hours=1)
        if start is None:
            s = end - timedelta(days=1)
        else:
            if start.tzinfo is None:
                s = start.replace(tzinfo=pytz.UTC)
            else:
                s = start.astimezone(pytz.UTC)
            s = s.replace(minute=0, second=0)
        return (s,end)

    @staticmethod
    def fetchHourRange(start,end):
        s = DatetimeUtility.toUnixTs(dt=start)
        e = DatetimeUtility.toUnixTs(dt=end)
        r = int(s) % 3600
        if r > 0:
            s = s - r
        r = int(e) % 3600
        if r > 0:
            e = e +(3600-r)
        return (s,e)

    @staticmethod
    #根据freq生成index
    def time_range(start,end,freq):
        if freq == 'M':
            f = 'BM'
        else:
            f = freq
        index = pd.date_range(start,end,freq=f,closed='left')
        return index


    @staticmethod
    def fetchStatRange(start,end):
        s = DatetimeUtility.toUnixTs(dt=start)
        e = DatetimeUtility.toUnixTs(dt=end)
        r = int(s) % 3600
        if r > 0:
            s = s - r
        r = int(e) % 3600
        if r > 0:
            e = e +(3600-r)
        return (s,e)

if __name__ == '__main__':
    (s,e)= DatetimeUtility.calcLast2Now()
    print("%s~%s" % (s,e))
    (s,e)= DatetimeUtility.calcLast24Hours()
    print("%s~%s" % (s,e))
    (start, end) = DatetimeUtility.fetchStatRange(s, datetime.now(pytz.UTC))
    print(start)
    print(end)
    e = int(time.time())
    r = int(e) % 3600
    if r > 0:
        e = e + (3600 - r)
    s = e - 86400
    print(e)
    print(s)
    #(start, end) = DatetimeUtility.fetchHourRange(s, e+timedelta(seconds=1))
    #print(start)
    #print(end)
    """
    (start,end) = DatetimeUtility.calcCurMonthRange2Ts()

    print('tzinfo: %s' % (DatetimeUtility.retrieveTimeoffset()))
    print("%d~%d" % (start,end))
    day0 = datetime.utcfromtimestamp(start)
    dayN = datetime.utcfromtimestamp(end)

    print('None: %s ~ %s' % (day0, dayN))
    print('None: %s ~ %s' % (day0, DatetimeUtility.toISO8601(dayN)))


    (start,end) = DatetimeUtility.calcCurMonthRange2Ts('Asia/Shanghai')
    print("%d~%d" % (start,end))
    day0 = datetime.fromtimestamp(start)
    dayN = datetime.fromtimestamp(end)
    print('8: %s ~ %s' % (day0, dayN))

    (start,end) = DatetimeUtility.calcCurMonthRange2Ts('Asia/Tokyo')
    print("%d~%d" % (start,end))
    day0 = datetime.utcfromtimestamp(start)
    dayN = datetime.utcfromtimestamp(end)

    print('10: %s ~ %s' % (day0, dayN))

    (start,end) = DatetimeUtility.calcCurMonthRange2Ts('US/Eastern')
    (s,e) = DatetimeUtility.calcCurMonthRange('US/Eastern')
    print("%d~%d" % (start,end))
    day0 = datetime.utcfromtimestamp(start)
    dayN = datetime.utcfromtimestamp(end)

    print('-5: %s ~ %s' % (day0,dayN))
    print('-5: %s ~ %s' % (s.replace(tzinfo=pytz.UTC),e))

    (start,end) = DatetimeUtility.calcCurMonthRange2Ts('UTC')
    print("%d~%d" % (start,end))
    day0 = datetime.utcfromtimestamp(start)
    dayN = datetime.utcfromtimestamp(end)

    print('0: %s ~ %s' % (day0,dayN))
    print(day0)

    (start,end) = DatetimeUtility.calcCurMonthRange('UTC')
    #period = DatetimeUtility.date_range('Asia/Shanghai',start,end)
    #print(period)
    print('-1---: %s ~ %s' % (start,end))
    #(start,end) = DatetimeUtility.calcTheDayRange('UTC','2018-10-02')


    #dateframe = pd.DataFrame(columns=['start_location','start_ts',"end_location","end_ts"],index=period)
    #print(dateframe)
    #print('2---: %s ~ %sZ' % (DatetimeUtility.toISO8601(start), end.isoformat()))

    to=DatetimeUtility.retrieveTimeoffset('US/Eastern')
    print(to)

    #print(DatetimeUtility.retrieveTimeZone(116.32672499993,40.076496))
    print("----------------1----------------")
    start = datetime.now(pytz.timezone("US/Eastern")) - timedelta(days=3)
    print(start)
    (s1,end) = DatetimeUtility.calcDateRange('Asia/Shanghai',start)

    print("%s ~ %s" %  (DatetimeUtility.toISO8601(s1),DatetimeUtility.toISO8601(end)))

    print("----------------2----------------")
    (s2,e2)=DatetimeUtility.calcTheDayRange(timezone="UTC",date_str="2018-12-12")
    print("%s ~ %s" %  (DatetimeUtility.toISO8601(s2),DatetimeUtility.toISO8601(e2)))

    print("----------------3----------------")
    (s3,e3)=DatetimeUtility.calcTheDayRange(date_str="2018-12-12")
    print("%s ~ %s" % (DatetimeUtility.toISO8601(s3),DatetimeUtility.toISO8601(e3)))

    print("----------------4----------------")

    (start,end) = DatetimeUtility.calcLast2Now()
    print("%s ~ %s" % (DatetimeUtility.toISO8601(start), DatetimeUtility.toISO8601(end)))


    print("----------------5----------------")
    dt = datetime.now(pytz.timezone("US/Eastern"))

    (start,end) = DatetimeUtility.calcLast2Now(start=dt)
    print("%s ~ %s" % (DatetimeUtility.toISO8601(start), DatetimeUtility.toISO8601(end)))

    print("----------------6----------------")
    start = datetime.now()
    print(start)
    (s1,end) = DatetimeUtility.calcDateRange('Asia/Shanghai',start)
    print("%s ~ %s" % (DatetimeUtility.toISO8601(s1), DatetimeUtility.toISO8601(end)))

    start=iso8601.parse_date("2019-01-01T00:00:00Z")
    end =iso8601.parse_date("2020-01-01T00:00:00Z")
    index=DatetimeUtility.time_range(start,end,"D")
    print(index)
    df = pd.DataFrame(columns=['v'],index =index)
    print(df)
    (s,d) = DatetimeUtility.calcTheDayRange("UTC","2019-01-31")
    newDf = pd.DataFrame({'v':[2,3]},index=[s,d])

    #print(newDf)
    
    for idx in newDf.index:
        df.ix[idx]['v']=newDf.ix[idx]['v']
    print(df)
    """
